package ru.uc_itcom.esd.view.vehicle;

import com.vaadin.flow.component.AbstractField;
import com.vaadin.flow.component.AttachEvent;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.textfield.TextField;
import com.vaadin.flow.router.QueryParameters;
import com.vaadin.flow.router.Route;
import io.jmix.core.Messages;
import io.jmix.flowui.component.textfield.TypedTextField;
import io.jmix.flowui.view.*;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import ru.uc_itcom.esd.entity.ParticipantVehicle;
import ru.uc_itcom.esd.entity.XmlInterface;
import ru.uc_itcom.esd.repository.unconstrain.ParticipantVehicleUcRepository;
import ru.uc_itcom.esd.utils.ViewHelper;
import ru.uc_itcom.esd.view.main.MainView;
import ru.uc_itcom.esd.view.trailer.ParticipantTrailerListView;
import ru.uc_itcom.esd.view.view_extension.DraftXmlDetailView;

import java.util.Objects;
import java.util.Optional;

@Route(value = "participant-vehicles/:id", layout = MainView.class)
@ViewController("ParticipantVehicle.detail")
@ViewDescriptor("participant-vehicle-detail-view.xml")
@EditedEntityContainer("participantVehicleDc")
public class ParticipantVehicleDetailView extends StandardDetailView<ParticipantVehicle> {

    @Autowired
    private ParticipantVehicleUcRepository participantVehicleUcRepository;

    @Autowired
    private Messages messages;

    @ViewComponent
    private TextField inventoryNumberField;

    @ViewComponent
    private TextField registrationNumberField;

    @Subscribe
    public void onAttachEvent(final AttachEvent event) {
        if (ObjectUtils.isEmpty(getEditedEntity().getParticipant())) {
            Optional.ofNullable(ViewHelper.findComponentByClass(Objects.requireNonNull(UI.getCurrent().getElement().getComponent().orElse(null)), DraftXmlDetailView.class))
                    .map(DraftXmlDetailView.class::cast)
                    .ifPresent(view -> getEditedEntity().setParticipant(((XmlInterface) view.getEditedEntity()).getParticipant()));
        }
    }

    @Subscribe("registrationNumberField")
    public void onRegistrationNumberFieldComponentValueChange(final AbstractField.ComponentValueChangeEvent<TypedTextField<String>, String> event) {
        String currentRegistrationNumber = getEditedEntity().getRegistrationNumber();
        if (StringUtils.isNotEmpty(event.getValue()) && (currentRegistrationNumber == null || !currentRegistrationNumber.equals(event.getValue()))) {
            if (participantVehicleUcRepository.existsByRegistrationNumber(event.getValue())) {
                registrationNumberField.setErrorMessage(event.getValue() + " " + messages.getMessage("excel.error.registration.number.is.exists"));
                registrationNumberField.setInvalid(true);
                registrationNumberField.clear();
            } else {
                registrationNumberField.setInvalid(false);
            }
        }
        inventoryNumberField.setRequired(event.getSource().isEmpty());
    }

    @Subscribe("inventoryNumberField")
    public void onInventoryNumberFieldComponentValueChange(final AbstractField.ComponentValueChangeEvent<TypedTextField<String>, String> event) {
        registrationNumberField.setRequired(event.getSource().isEmpty());
    }
}